# This script was generated by crazy-complete.
# crazy-complete: A tool that creates robust and reliable autocompletion scripts for Bash, Fish and Zsh.
# For more information, visit: https://github.com/crazy-complete/crazy-complete

function _nbfc_service_fish_query
  # ===========================================================================
  #
  # This function implements the parsing of options and positionals in the Fish shell.
  #
  # Usage: __fish_query <OPTIONS> <COMMAND> [ARGS...]
  #
  # The first argument is a comma-separated list of options that the parser should know about.
  # Short options (-o), long options (--option), and old-style options (-option) are supported.
  #
  # If an option takes an argument, it is suffixed by '='.
  # If an option takes an optional argument, it is suffixed by '=?'.
  #
  # For example:
  #   __fish_query '-f,--flag,-old-style,--with-arg=,--with-optional=?' [...]
  #
  #   Here, -f, --flag and -old-style don't take options, --with-arg requires an
  #   argument and --with-optional takes an optional argument.
  #
  # COMMANDS
  #   positional_contains <NUM> <WORDS...>
  #     Checks if the positional argument number NUM is one of WORDS.
  #     NUM counts from one.
  #
  #   has_option <OPTIONS...>
  #     Checks if an option given in OPTIONS is passed on commandline.
  #
  #   option_is <OPTIONS...> -- <VALUES...>
  #     Checks if any option in OPTIONS has a value of VALUES.
  #
  #   num_of_positionals [<OPERATOR> <NUMBER>]
  #     Checks the number of positional arguments.
  #     If no arguments are provided, print the total count of positional arguments.
  #     If two arguments are provided, the first argument should be one of
  #     the comparison operators: '-lt', '-le', '-eq', '-ne', '-gt', '-ge'.
  #     Returns 0 if the count of positional arguments matches the
  #     specified NUMBER according to the comparison operator, otherwise returns 1.
  #
  # ===========================================================================

  set -l func '__fish_query'

  set -l positionals
  set -l having_options
  set -l option_values

  switch (count $argv)
    case 0
      echo "$func: missing OPTIONS argument" >&2
      return 1
    case 1
      echo "$func: missing COMMAND" >&2
      return 1
  end

  set -l options $argv[1]
  set -e argv[1]

  set -l cmd $argv[1]
  set -e argv[1]

  set -l my_cache_key "$(commandline -b) $options"

  if test "$__QUERY_CACHE_KEY" = "$my_cache_key"
    set positionals    $__QUERY_CACHE_POSITIONALS
    set having_options $__QUERY_CACHE_HAVING_OPTIONS
    set option_values  $__QUERY_CACHE_OPTION_VALUES
  else
    # =========================================================================
    # Parsing of OPTIONS argument
    # =========================================================================

    set -l short_opts_with_arg
    set -l long_opts_with_arg
    set -l old_opts_with_arg

    set -l short_opts_without_arg
    set -l long_opts_without_arg
    set -l old_opts_without_arg

    set -l short_opts_with_optional_arg
    set -l long_opts_with_optional_arg
    set -l old_opts_with_optional_arg

    set -l option

    if test -n "$options"
      for option in (string split -- ',' $options)
        # Using one big switch case is the fastest way
        switch $option
          case '--?*=';   set -a long_opts_with_arg           (string replace -- '='  '' $option)
          case '--?*=\?'; set -a long_opts_with_optional_arg  (string replace -- '=?' '' $option)
          case '--?*';    set -a long_opts_without_arg        $option

          case '-?=';     set -a short_opts_with_arg          (string replace -- '='  '' $option)
          case '-?=\?';   set -a short_opts_with_optional_arg (string replace -- '=?' '' $option)
          case '-?';      set -a short_opts_with_arg          $option

          case '-??*=';   set -a old_opts_with_arg            (string replace -- '='  '' $option)
          case '-??*=\?'; set -a old_opts_with_optional_arg   (string replace -- '=?' '' $option)
          case '-??*';    set -a old_opts_without_arg         $option

          case '*'
            echo "$func: argv[1]: '$option' is not a short, long or old-style option" >&2
            return 1
        end
      end
    end

    # =========================================================================
    # Parsing of options and positionals
    # =========================================================================

    set -l cmdline (commandline -poc)
    set -l cmdline_count (count $cmdline)

    set -l argi 2 # cmdline[1] is command name
    while test $argi -le $cmdline_count
      set -l arg "$cmdline[$argi]"
      set -l have_trailing_arg (test $argi -lt $cmdline_count && echo true || echo false)

      switch $arg
        case '-'
          set -a positionals -
        case '--'
          for argi in (seq (math $argi + 1) $cmdline_count)
            set -a positionals $cmdline[$argi]
          end
          break
        case '--*=*'
          set -l split (string split -m 1 -- '=' $arg)
          set -a having_options $split[1]
          set -a option_values "$split[2]"
        case '--*'
          if contains -- $arg $long_opts_with_arg
            if $have_trailing_arg
              set -a having_options $arg
              set -a option_values $cmdline[(math $argi + 1)]
              set argi (math $argi + 1)
            end
          else
            set -a having_options $arg
            set -a option_values ""
          end
        case '-*'
          set -l end_of_parsing false

          if string match -q -- "*=*" $arg
            set -l split (string split -m 1 -- '=' $arg)
            if contains -- $split[1] $old_opts_with_arg $old_opts_with_optional_arg
              set -a having_options $split[1]
              set -a option_values "$split[2]"
              set end_of_parsing true
            end
          else if contains -- $arg $old_opts_with_arg
            set end_of_parsing true
            if $have_trailing_arg
              set -a having_options $arg
              set -a option_values $cmdline[(math $argi + 1)]
              set argi (math $argi + 1)
            end
          else if contains -- $arg $old_opts_without_arg $old_opts_with_optional_arg
            set -a having_options $arg
            set -a option_values ""
            set end_of_parsing true
          end

          set -l arg_length (string length -- $arg)
          set -l i 2
          while not $end_of_parsing; and test $i -le $arg_length
            set -l option "-$(string sub -s $i -l 1 -- $arg)"
            set -l trailing_chars "$(string sub -s (math $i + 1) -- $arg)"

            if contains -- $option $short_opts_without_arg
              set -a having_options $option
              set -a option_values ""
            else if contains -- $option $short_opts_with_arg
              set end_of_parsing true

              if test -n "$trailing_chars"
                set -a having_options $option
                set -a option_values $trailing_chars
              else if $have_trailing_arg
                set -a having_options $option
                set -a option_values $cmdline[(math $argi + 1)]
                set argi (math $argi + 1)
              end
            else if contains -- $option $short_opts_with_optional_arg
              set -a having_options $option
              set end_of_parsing true
              set -a option_values "$trailing_chars" # may be empty
            end

            set i (math $i + 1)
          end
        case '*'
          set -a positionals $arg
      end

      set argi (math $argi + 1)
    end

    set -g __QUERY_CACHE_POSITIONALS    $positionals
    set -g __QUERY_CACHE_HAVING_OPTIONS $having_options
    set -g __QUERY_CACHE_OPTION_VALUES  $option_values
    set -g __QUERY_CACHE_KEY            $my_cache_key
  end

  # ===========================================================================
  # Commands
  # ===========================================================================

  switch $cmd
    case 'positional_contains'
      if test (count $argv) -eq 0
        echo "$func: positional_contains: argv[3]: missing number" >&2
        return 1
      end

      set -l positional_num $argv[1]
      set -e argv[1]
      contains -- $positionals[$positional_num] $argv && return 0 || return 1
    case 'has_option'
      for option in $having_options
        contains -- $option $argv && return 0
      end

      return 1
    case 'num_of_positionals'
      switch (count $argv)
        case 0
          count $positionals
        case 1
          echo "$func: num_of_positionals: $argv[1]: missing operand" >&2
          return 1
        case 2
          if contains -- $argv[1] -lt -le -eq -ne -gt -ge;
            test (count $positionals) $argv[1] $argv[2] && return 0 || return 1
          else
            echo "$func: num_of_positionals: $argv[1]: unknown operator" >&2
            return 1
          end
        case '*'
          echo "$func: num_of_positionals: too many arguments" >&2
          return 1
      end
    case 'option_is'
      set -l options
      set -l values

      for arg in $argv
        set -e argv[1]

        if string match -q -- -- $arg
          set values $argv
          break
        else
          set -a options $arg
        end
      end

      if test (count $values) -eq 0
        echo "$func: missing values" >&2
        return 1
      end

      set -l i (count $having_options)
      while test $i -ge 1
        if contains -- $having_options[$i] $options
          if contains -- $option_values[$i] $values
            return 0
          end
        end

        set i (math $i - 1)
      end

      return 1
    case '*'
      echo "$func: argv[2]: invalid command" >&2
      return 1
  end
end

set -l prog "nbfc_service"
set -l query "_nbfc_service_fish_query"

# Generally disable file completion
complete -c $prog -x

# command nbfc_service
set -l opts "-h,--help,-r,--read-only,-f,--fork,-d,--debug,-c=,--config-file=,-e=,--embedded-controller="
set -l C000 "not $query '$opts' has_option -h --help"
set -l C001 "not $query '$opts' has_option -r --read-only"
set -l C002 "not $query '$opts' has_option -f --fork"
set -l C003 "not $query '$opts' has_option -d --debug"
set -l C004 "not $query '$opts' has_option -c --config-file"
set -l C005 "not $query '$opts' has_option -e --embedded-controller"
complete -c $prog -n $C000 -s h -l help -d 'show this help message and exit' -f
complete -c $prog -n $C001 -s r -l read-only -d 'Start in read-only mode' -f
complete -c $prog -n $C002 -s f -l fork -d 'Switch process to background after sucessfully started' -f
complete -c $prog -n $C003 -s d -l debug -d 'Enable tracing of reads and writes of the embedded controller' -f
complete -c $prog -n $C004 -s c -l config-file -d 'Use alternative config file (default /etc/nbfc/nbfc.json)' -rF
complete -c $prog -n $C005 -s e -l embedded-controller -d 'Specify embedded controller to use' -x -a 'dummy dev_port ec_sys acpi_ec'

# vim: ft=fish ts=2 sts=2 sw=2 et
